package persistence;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import persistence.category.CategoryEntity;

import java.io.File;
import java.io.IOException;

import static org.junit.jupiter.api.Assertions.assertEquals;

class GenericCSVDAOTest {

	GenericCSVDAO<CategoryEntity> sut;
	File file = new File("test.csv");

	@BeforeEach
	public void beforeEach() throws IOException {

		if (file.exists()) {
			file.delete();
		}
		file.createNewFile();

		this.sut = new GenericCSVDAO<>(file, CategoryEntity::new);
	}

	@AfterEach
	void afterEach() {
		file.delete();
	}

	@Test
	public void addWorks() throws IOException {
		assertEquals(0, sut.getALl().size());

		var entityToAdd = new CategoryEntity("categoryName", 99);
		sut.add(entityToAdd);
		var foundEntity = sut.getALl().stream().findFirst().orElseThrow();

		assertEquals(entityToAdd, foundEntity);
	}

	@Test
	public void removeWorks() throws IOException {
		var entityToAdd = new CategoryEntity("categoryName", 99);
		sut.add(entityToAdd);

		sut.remove(entityToAdd);

		assertEquals(0, sut.getALl().size());
	}

	@Test
	public void removeAllWorks() throws IOException {
		var entityToAdd1 = new CategoryEntity("categoryName1", 101);
		sut.add(entityToAdd1);
		var entityToAdd2 = new CategoryEntity("categoryName2", 102);
		sut.add(entityToAdd2);
		assertEquals(2, sut.getALl().size());
		sut.removeAll();

		assertEquals(0, sut.getALl().size());
	}
}
