package link;

import category.CategoryName;
import category.CategoryRepository;
import exeptions.URLIsNotReachable;
import tag.TaggingUseCase;
import user.Username;

import java.util.Set;
import java.util.stream.Collectors;

public class LinkUseCase {

	private final LinkRepository linkRepository;
	private final CategoryRepository categoryRepository;
	private final TaggingUseCase taggingUseCase;
	private final LinkIdGenerator linkIdGenerator;

	public LinkUseCase(LinkRepository linkRepository,
	                   CategoryRepository categoryRepository,
	                   TaggingUseCase taggingUseCase,
	                   LinkIdGenerator linkIdGenerator) {
		this.linkRepository = linkRepository;
		this.categoryRepository = categoryRepository;
		this.taggingUseCase = taggingUseCase;
		this.linkIdGenerator = linkIdGenerator;
	}

	public void addLink(LinkUrl url, Set<CategoryName> categoryNames, Username creator) {

		if (!OnlineCheck.isReachable(url)) {
			throw new URLIsNotReachable("The url " + url + " does not seem online. Check if it is correct and you have internet access.");
		}

		var categoryIds = categoryNames.stream().map(categoryRepository::getIdByName).collect(Collectors.toSet());

		var tags = taggingUseCase.getTagsFor(url);

		var id = linkIdGenerator.generateId();

		var link = new Link(id, creator, url, categoryIds, tags);

		linkRepository.add(link);
	}

}
